'use strict';

app.controller('ApplicationCtrl', ['$rootScope', '$scope', '$templateCache', '$interval', '$localStorage', 'applicationData', 'toaster', 'FormDataService',
    function ($rootScope, $scope, $templateCache, $interval, $localStorage, applicationData, toaster, FormDataService) {
	
    $rootScope.applicationData = applicationData;

    $templateCache.put('mediaNotification.html', 
      '<div class="messageContainer">' +
        '<div class="thumbnailContainer hidden-xs">' +
          '<img src="assets/images/thumbnail-shape.png" class="thumbnailImg" style="background: url({{toaster.data.thumbnailUrl}}) 50% 50% no-repeat;">' +
        '</div>' +
        '<div class="description">' +
          '<div class="toast-title">{{"notification_new_file_added" | translate}}</div>' +
          '<div class="title">{{toaster.data.title}}</div>' +
          '<div>{{toaster.data.fileName}}</div>' +
        '</div>' +
      '</div>');

    $templateCache.put('playbackNotification.html', 
      '<div class="messageContainer">' +
        '<div class="thumbnailContainer hidden-xs">' +
          '<img src="assets/images/thumbnail-shape.png" class="thumbnailImg" style="background: url({{toaster.data.thumbnailUrl}}) 50% 50% no-repeat;">' +
        '</div>' +
        '<div class="description">' +
          '<div class="toast-title">{{"notification_playback_" + toaster.data.status | translate}}</div>' +
          '<div class="title">{{toaster.data.title}}</div>' +
          '<div>{{toaster.data.device}}</div>' +
        '</div>' +
      '</div>');

    
    var notificationToastDurationMs = 3000;

    var pullFileUpdatesIntervalMs = 5000;
    var pullFileUpdatesTask;
    var previouslyDisplayedFileIdNotification = null;

    $scope.startPullingFileUpdates = function() {
      if ( angular.isDefined(pullFileUpdatesTask) ) return;

      pullFileUpdatesTask = $interval(function() {

      	FormDataService.loadFormData('library-status', true).then(function(json) {
          if(json != null) {
        		$rootScope.$broadcast('fileTotalsUpdated', json.totalAudioFiles, json.totalVideoFiles, json.totalImageFiles);

            if($localStorage.showFileNotifications) {
              var lastAddedFile = json.lastAddedFile;          
          		if(lastAddedFile) {
          			if(previouslyDisplayedFileIdNotification != lastAddedFile.mediaItemId) {
                  var templateData = {
                    thumbnailUrl: lastAddedFile.thumbnailUrl,
                    fileName: lastAddedFile.fileName,
                    title: lastAddedFile.title 
                  };
                  toaster.pop('media', '', "{template: 'mediaNotification.html', data: " + angular.toJson(templateData) + "}", notificationToastDurationMs, 'templateWithData');
          				previouslyDisplayedFileIdNotification = lastAddedFile.mediaItemId;        	
          			}  				
          		}
            }
          }
      	});                
      }, pullFileUpdatesIntervalMs);
    };

	  $scope.stopPullingFileUpdates = function() {
      if (angular.isDefined(pullFileUpdatesTask)) {
        $interval.cancel(pullFileUpdatesTask);
        pullFileUpdatesTask = undefined;
      }
    };

    var pullPlaybackUpdatesIntervalMs = 5000;
    var playbackStatusUpdatesTask;
    var lastPlaybackStatus;
  
    $scope.startPullingPlaybackUpdates = function() {
      if ( angular.isDefined(playbackStatusUpdatesTask) ) return;

      playbackStatusUpdatesTask = $interval(function() {
        if($localStorage.showPlaybackNotifications) {
          FormDataService.loadFormData('playback-status', true).then(function(json) {
            if(json != null) {            
              var playbackStatus = json.clients;     
              if(!angular.isDefined(lastPlaybackStatus)) {
                // the first call after start, just save the state
                lastPlaybackStatus = playbackStatus;
              } else {
                var startedFiles = playbackStatus.filter(function(client) {
                  var previousClientState = lastPlaybackStatus.find(function(lastClient) {
                    return lastClient.clientIPAddress == client.clientIPAddress;
                  });
                  return previousClientState == null || previousClientState.itemId != client.itemId;
                });

                var stoppedFiles = lastPlaybackStatus.filter(function(lastClient) {
                  var newClientState = playbackStatus.find(function(newClient) {
                    return newClient.clientIPAddress == lastClient.clientIPAddress;
                  });
                  return newClientState == null || newClientState.itemId != lastClient.itemId;
                });

                lastPlaybackStatus = playbackStatus;

                var invokeToastr = function(started) {
                  return function(item) {
                    var templateData = {
                      thumbnailUrl: item.itemThumbnailUrl,
                      device: item.clientName,
                      title: item.itemTitle,
                      status: started ? 'started' : 'stopped' 
                    };
                    toaster.pop('playback', '', "{template: 'playbackNotification.html', data: " + angular.toJson(templateData) + "}", notificationToastDurationMs, 'templateWithData'); 
                  }
                }; 

                // only show stop event if there is no immediate start event on the same client, otherwise only show the start event  
                stoppedFiles = stoppedFiles.filter(function(stoppedItem) {
                  return startedFiles.find(function(startedItem) {
                    return startedItem.clientIPAddress == stoppedItem.clientIPAddress;
                  }) == null;
                });

                stoppedFiles.forEach(invokeToastr(false));
                startedFiles.forEach(invokeToastr(true));                
              }            
            }
          });
        }                
      }, pullPlaybackUpdatesIntervalMs);
    };

    $scope.stopPullingPlaybackUpdates = function() {
      if (angular.isDefined(playbackStatusUpdatesTask)) {
        $interval.cancel(playbackStatusUpdatesTask);
        playbackStatusUpdatesTask = undefined;
      }
    };

     
    $scope.$on('$destroy', function() {
      $scope.stopPullingFileUpdates();
      $scope.stopPullingPlaybackUpdates();
    });

    // kick off the library status pulling
    $scope.startPullingFileUpdates();
    // kick off the playback status
    $scope.startPullingPlaybackUpdates();

}]);